
#include "arf.h"
#include <stdio.h>

Arf::Arf()
{
  currentRate = 0;
  consecutiveSuccess = 0;
  consecutiveFailures = 0;
  probationNumber = 0;
}

void
Arf::successfulTransmission ()
{
  // if still in probation
  if (probationNumber > 0){
    // Correctly tx, so leave probation
    probationNumber = 0;
  }
  else {
    consecutiveSuccess++;
    consecutiveFailures = 0;
    
    if (consecutiveSuccess >= STEPUP_THRESHOLD){
      // don't increase beyond the Max
      if (currentRate < 3)
	currentRate++;
      // reset counters
      consecutiveSuccess = 0;
      consecutiveFailures = 0;
      // enter probation
      probationNumber = PROBATION_SIZE;
    }
    
  }
}

void
Arf::failedTransmission ()
{
  // Check for failed last chance in probation
  if (probationNumber == 1){
    currentRate--;
    consecutiveSuccess = 0;
    consecutiveFailures = 0;
    probationNumber = 0;    
  }
  else if (probationNumber > 1){
    // failed a chance to get out of probation
    probationNumber--;
  }
  else{
    consecutiveSuccess = 0;
    consecutiveFailures++;
    // Too many failures, fallback
    if ((consecutiveFailures >= FALLBACK_THRESHOLD) && (currentRate > 0)){
      currentRate--;
      consecutiveSuccess = 0;
      consecutiveFailures = 0;
      probationNumber = 0;
    }
  }
  
}


double
Arf::getTransmissionRate ()
{
  double rate;
  switch (currentRate){
  case 0:
    rate = 1000000.0;
    break;
  case 1:
    rate = 2000000.0;
    break;
  case 2:
    rate = 5500000.0;
    break;
  case 3:
    rate = 11000000.0;
    break;
  default:
    printf("Unknown rate %d\n", currentRate);
    rate = 0;
  }

  return rate;
}
