/* -*- c++ -*-
   packet-stamp.h
   $Id: packet-stamp.h,v 1.3 1999/03/13 03:52:58 haoboy Exp $

   Information carried by a packet to allow a receive to decide if it
   will recieve the packet or not.

*/

#ifndef _cmu_packetstamp_h_
#define _cmu_packetstamp_h_

class MobileNode;
/* to avoid a pretty wild circular dependence among header files
   (between packet.h and queue.h), I can't do the #include here:
   #include <cmu/node.h>
   Since PacketStamp is just a container class, it doesn't really matter .
   -dam 8/8/98
   */
#include <antenna.h>

class PacketStamp {
public:
  //Modified Kevin
  PacketStamp() : ant(0), node(0), Pr(-1), lambda(-1), noise(0), noise_value(0) { }

  void init(const PacketStamp *s) {
	  Antenna* ant;
	  if (s->ant != NULL)
		  ant = s->ant->copy();
	  else
		  ant = 0;

	  //Antenna *ant = (s->ant) ? s->ant->copy(): 0;
	  //Modified Kevin
	  stamp(s->node, ant, s->Pr, s->lambda, s->channel);
	  noise = s->noise;
  }

  //Changed Kevin
  // to stamp the interface
  void stamp(MobileNode *n, Antenna *a, double xmitPr, double lam, int chan) {
    ant = a;
    node = n;
    Pr = xmitPr;
    lambda = lam;
    // Added Kevin
    channel = chan;
  }

  // Added Kevin
  inline int getChannel() {return channel;}
  inline Antenna * getAntenna() {return ant;}
  inline MobileNode * getNode() {return node;}
  inline double getTxPr() {return Pr;}
  inline double getLambda() {return lambda;}
  // Added Kevin
  inline double getRxPr() {return RxPr;}

  /* WILD HACK: The following two variables are a wild hack.
     They will go away in the next release...
     They're used by the mac-802_11 object to determine
     capture.  This will be moved into the net-if family of 
     objects in the future. */
  double RxPr;			// power with which pkt is received
  double CPThresh;		// capture threshold for recving interface
  // Added Kevin Begin
  bool     noise;          // use very basic SNR calculation
                           // Packet is sent to every interface, only the nearest interface processes the data and ack packets
  double noise_value; //noise present at the time the packet began arriving
  //double recv_pw; //power at the interface when the packet began arriving

  // Added Kevin End

protected:
  Antenna       *ant;
  MobileNode	*node;
  double        Pr;		// power pkt sent with
  double        lambda;         // wavelength of signal
  // Added Kevin Begin
  int      channel;        // Channel this packet is traversing
                           // Could not do a pointer to channel due to crazy include dependencies
  // Added Kevin End
};

#endif /* !_cmu_packetstamp_h_ */
