proc UniformErr {} {
    set err [new ErrorModel/CL]
    $err unit packet
    return $err
}


Node/MobileNode set default_error_ .00
Node/MobileNode set max_channels_ 12
Node/MobileNode set switch_time_ .001

#power and noise values taken from "Efficient Geographic Routing in Multihop Wireless Networks", S. Lee, B. Bhatacharjee, S. Banerjee
Phy/WirelessPhy set ambient_noise_ .0000000000012
Phy/WirelessPhy set Pt_ 0.020
Phy/WirelessPhy set CSThresh_ 10

# 24 bytes sent at 1Mbps
Mac/802_11 set PreambleLength_ 144

#clear_rt_on_switch indicates if the routing tables should be cleared when an interface changes channels. 1 = clear routing tables, 0 = leave routing tables
Node/MobileNode set clear_rt_on_switch_         1;



set val(chan)           Channel/WirelessChannel    ;# Channel Type
set val(prop)           Propagation/TwoRayGround   ;# radio-propagation model
set val(netif)          Phy/WirelessPhy            ;# network interface type
set val(mac)            Mac/802_11                 ;# MAC type
set val(ifq)            Queue/DropTail/PriQueue    ;# interface queue type
set val(ll)             LL                         ;# link layer type
set val(ant)            Antenna/OmniAntenna        ;# antenna model
set val(ifqlen)         50                         ;# max packet in ifq
set val(nn)             6                          ;# number of mobilenodes
set val(rp)             AODV                  ;# routing protocol
set val(x)		800
set val(y)		800
set val(interfaces)     1                    ;#number of interfaces each node has
set val(length)         800                  ;#length of simulation


# Initialize Global Variables
set ns_		[new Simulator]
set tracefd     [open ./example2.tr w]
#$ns_ use-newtrace
$ns_ trace-all $tracefd


# set up topography object
set topo       [new Topography]

$topo load_flatgrid $val(x) $val(y)

# Create God
create-god [expr $val(nn) * ($val(interfaces) + 1)]

# Create channels
#notice how channels are created ahead of time
for {set i 0} {$i < 15 } {incr i} {
    set chan($i) [new $val(chan)] 
}


#=====================================	
#must set ErrorModel/CL to get gray model
set inerrProc_ "UniformErr"
set outerrProc_ ""
set FECProc_ ""
set propInstance_ [new $val(prop)]
set port [Node set rtagent_port_]


# configure node, please note the change below.
$ns_ node-config -adhocRouting $val(rp) \
		-llType $val(ll) \
		-macType $val(mac) \
		-ifqType $val(ifq) \
		-ifqLen $val(ifqlen) \
		-antType $val(ant) \
		-propType $val(prop) \
		-phyType $val(netif) \
		-topoInstance $topo \
		-agentTrace ON \
		-routerTrace OFF \
		-macTrace OFF \
		-movementTrace OFF \
		-channel $chan(0) \
                -IncomingErrProc UniformErr


for {set i 0} {$i < $val(nn) } {incr i} {
    # create the node
    set node_($i) [$ns_ node]  
    # get the routing agent
    set agent [$node_($i) set ragent_] 
    for {set j 0} {$j < $val(interfaces) } {incr j} {
	# add an interface on the next channel
	$node_($i) add-interface $chan([expr {$j + 1}]) $propInstance_ $val(ll) $val(mac)	\
	    $val(ifq) $val(ifqlen) $val(netif) $val(ant) \
	    $topo $inerrProc_ $outerrProc_ $FECProc_
	#this is stupid and redundent, but I'm not going to spend the time rewritting it
	$node_($i) adjust-target-rtagent $agent $port
    }
    $node_($i) random-motion 0
    $node_($i) enable-arf
}

# Place nodes
$node_(0) set X_ 50.0
$node_(0) set Y_ 100.0
$node_(0) set Z_ .9

$node_(1) set X_ 50.0
$node_(1) set Y_ 250.0
$node_(1) set Z_ .9

$node_(2) set X_ 730.0
$node_(2) set Y_ 100.0
$node_(2) set Z_ .9

$node_(3) set X_ 730.0
$node_(3) set Y_ 300.0
$node_(3) set Z_ .9

$node_(4) set X_ 50.0
$node_(4) set Y_ 100.0
$node_(4) set Z_ .9

$node_(5) set X_ 50.0
$node_(5) set Y_ 350.0
$node_(5) set Z_ .9

# Create transmissions
set tcp1 [new Agent/TCP]
set sink1 [new Agent/TCPSink]
$node_(0) attach $tcp1 30 1
$node_(1) attach $sink1 30 1
$ns_ connect $tcp1 $sink1
set ftp1 [new Application/FTP]
$ftp1 attach-agent $tcp1

set tcp2 [new Agent/TCP]
set sink2 [new Agent/TCPSink]
$node_(2) attach $tcp2 30 1
$node_(3) attach $sink2 30 1
$ns_ connect $tcp2 $sink2
set ftp2 [new Application/FTP]
$ftp2 attach-agent $tcp2

set tcp3 [new Agent/TCP]
set sink3 [new Agent/TCPSink]
$node_(4) attach $tcp3 30 1
$node_(5) attach $sink3 30 1
$ns_ connect $tcp3 $sink3
set ftp3 [new Application/FTP]
$ftp3 attach-agent $tcp3


$ns_ at 10.1 "$ftp1 produce 700"
$ns_ at 11.5 "puts \"\\nFirst demonstrate ARF as distance increases\""
$ns_ at 11.5 "puts \"150m transmission, 1st sample: \[$node_(0) dump-arf-rate 1\]\""
$ns_ at 12.0 "puts \"1500m transmission, 2nd sample: \[$node_(0) dump-arf-rate 1\]\""
$ns_ at 13.0 "puts \"150m transmission, 3rd sample: \[$node_(0) dump-arf-rate 1\]\\n\""

$ns_ at 20.1 "$ftp2 produce 500"
$ns_ at 22.5 "puts \"200m transmission, 1st sample: \[$node_(2) dump-arf-rate 1\]\""
$ns_ at 23.0 "puts \"200m transmission, 2nd sample: \[$node_(2) dump-arf-rate 1\]\""
$ns_ at 24.0 "puts \"200m transmission, 3rd sample: \[$node_(2) dump-arf-rate 1\]\\n\""

$ns_ at 30.1 "$ftp3 produce 400"
$ns_ at 34.0 "puts \"250m transmission, 1st sample: \[$node_(4) dump-arf-rate 1\]\""
$ns_ at 36.0 "puts \"250m transmission, 2nd sample: \[$node_(4) dump-arf-rate 1\]\""
$ns_ at 38.0 "puts \"250m transmission, 3rd sample: \[$node_(4) dump-arf-rate 1\]\\n\""

$ns_ at 50.0 "puts \"\\nNow show ARF with interference\""
$ns_ at 50.1 "$ftp2 produce 5000"
$ns_ at 52.5 "puts \"200m transmission(without interference), 1st sample: \[$node_(2) dump-arf-rate 1\]\""
$ns_ at 53.0 "puts \"200m transmission(without interference), 2nd sample: \[$node_(2) dump-arf-rate 1\]\""
$ns_ at 54.0 "puts \"200m transmission(without interference), 3rd sample: \[$node_(2) dump-arf-rate 1\]\\n\""

$ns_ at 60.1 "$ftp1 produce 3000"
$ns_ at 62.5 "puts \"200m transmission(with interference), 1st sample: \[$node_(2) dump-arf-rate 1\]\""
$ns_ at 63.0 "puts \"200m transmission(with interference), 2nd sample: \[$node_(2) dump-arf-rate 1\]\""
$ns_ at 64.0 "puts \"200m transmission(with interference), 3rd sample: \[$node_(2) dump-arf-rate 1\]\\n\""

$ns_ at 75.0 "puts \"200m transmission(without interference), 1st sample: \[$node_(2) dump-arf-rate 1\]\""
$ns_ at 76.5 "puts \"200m transmission(without interference), 2nd sample: \[$node_(2) dump-arf-rate 1\]\""
$ns_ at 78.0 "puts \"200m transmission(without interference), 3rd sample: \[$node_(2) dump-arf-rate 1\]\\n\""


$ns_ at $val(length) "stop"
$ns_ at [expr {$val(length) +.0001}] "puts \"NS EXITING...\" ; $ns_ halt"

proc stop {} {
    global ns_ tracefd
    $ns_ flush-trace
    close $tracefd
}

puts "Starting Simulation..."
$ns_ run
